package admin

import (
	"strconv"

	"github.com/dmitriygnatenko/internal/helpers"
	"github.com/dmitriygnatenko/internal/interfaces"
	"github.com/dmitriygnatenko/internal/mapper"
	"github.com/dmitriygnatenko/internal/models"
	"github.com/go-playground/validator/v10"
	"github.com/gofiber/fiber/v2"
)

const errTagExists = "Тег с данным URL уже существует"

func TagHandler(sp interfaces.IServiceProvider) fiber.Handler {
	return func(ctx *fiber.Ctx) error {
		tags, err := sp.GetTagRepository().GetAll(ctx.Context())
		if err != nil {
			return err
		}

		tagsDTO := mapper.ConvertTagModelsToDTO(tags)

		return ctx.Render("admin/tag", fiber.Map{
			"tags":    tagsDTO,
			"section": "tag",
		}, "admin/_layout")
	}
}

func AddTagHandler(sp interfaces.IServiceProvider) fiber.Handler {
	return func(ctx *fiber.Ctx) error {
		var form models.TagForm
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helpers.GetDefaultTranslator(validate)
		if err != nil {
			return err
		}

		if ctx.Method() == fiber.MethodPost {
			if err = ctx.BodyParser(&form); err != nil {
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helpers.FormatValidateErrors(err, trans)
			}

			if res, _ := sp.GetTagRepository().GetByURL(ctx.Context(), form.URL); res != nil {
				validateErrors["TagForm.URL"] = errTagExists
			}

			if len(validateErrors) == 0 {
				err = sp.GetTagRepository().Add(ctx.Context(), mapper.ConvertTagFormToModel(form))
				if err != nil {
					return err
				}

				sp.GetCacheService().FlushAll()

				if err = ctx.Redirect("/admin/tag"); err != nil {
					return err
				}
			}
		}

		return ctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   "Добавление тега",
		}, "admin/_layout")
	}
}

func EditTagHandler(sp interfaces.IServiceProvider) fiber.Handler {
	return func(ctx *fiber.Ctx) error {
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helpers.GetDefaultTranslator(validate)
		if err != nil {
			return err
		}

		ID, err := strconv.Atoi(ctx.Params("id"))
		if err != nil {
			return err
		}

		tag, err := sp.GetTagRepository().GetByID(ctx.Context(), ID)
		if err != nil {
			return err
		}

		form := models.TagForm{
			ID:  ID,
			Tag: tag.Tag,
			URL: tag.URL,
		}

		if ctx.Method() == fiber.MethodPost {
			if err = ctx.BodyParser(&form); err != nil {
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helpers.FormatValidateErrors(err, trans)
			}

			if res, _ := sp.GetTagRepository().GetByURL(ctx.Context(), form.URL); res != nil {
				if res.ID != ID {
					validateErrors["TagForm.URL"] = errTagExists
				}
			}

			if len(validateErrors) == 0 {
				err = sp.GetTagRepository().Update(ctx.Context(), mapper.ConvertTagFormToModel(form))
				if err != nil {
					return err
				}

				sp.GetCacheService().FlushAll()

				if err = ctx.Redirect("/admin/tag"); err != nil {
					return err
				}
			}
		}

		return ctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   "Редактирование тега",
		}, "admin/_layout")
	}
}

func DeleteTagHandler(sp interfaces.IServiceProvider) fiber.Handler {
	return func(ctx *fiber.Ctx) error {
		ID, err := strconv.Atoi(ctx.Params("id"))
		if err != nil {
			return err
		}

		tag, err := sp.GetTagRepository().GetByID(ctx.Context(), ID)
		if err != nil {
			return err
		}

		used, err := sp.GetTagRepository().IsUsed(ctx.Context(), ID)
		if err != nil {
			return err
		}

		if ctx.Method() == fiber.MethodPost {
			err = sp.GetTagRepository().Delete(ctx.Context(), ID)
			if err != nil {
				return err
			}

			sp.GetCacheService().FlushAll()

			if err = ctx.Redirect("/admin/tag"); err != nil {
				return err
			}
		}

		return ctx.Render("admin/tag_delete", fiber.Map{
			"tag":     tag.Tag,
			"used":    used,
			"section": "tag",
		}, "admin/_layout")
	}
}
